<?php
// $Id: bitcache.admin.inc,v 1.8 2008/10/01 08:33:38 arto Exp $

//////////////////////////////////////////////////////////////////////////////
// Bitcache settings

/**
 * Form builder: displays the Bitcache configuration screen.
 *
 * @ingroup forms
 * @see system_settings_form()
 */
function bitcache_admin_settings() {
  $form = array();

  // Server settings
  $form['server'] = array('#type' => 'fieldset', '#title' => t('Server settings'), '#collapsible' => TRUE, '#collapsed' => TRUE);
  $form['server']['bitcache_server'] = array('#type' => 'radios', '#title' => t('Bitcache REST API'), '#default_value' => (int)BITCACHE_SERVER, '#options' => array(t('Disabled'), t('Enabled')), '#description' => t('When enabled, the built-in Bitcache server is made available at !url. Access to the <a href="http://en.wikipedia.org/wiki/Representational_State_Transfer" target="_blank" title="Representational State Transfer">REST</a> <acronym title="Application Programming Interface">API</acronym> is subject to Drupal <a href="@manage-users">authentication</a> and <a href="@manage-perms">permissions</a>.', array('!url' => l(BITCACHE_URL, BITCACHE_URL), '@manage-users' => url('admin/user'), '@manage-perms' => url('admin/user/permissions', array('fragment' => 'module-bitcache')))));
  $form['server']['bitcache_features'] = array('#type' => 'checkboxes', '#title' => t('Enabled features'), '#default_value' => array_filter(variable_get('bitcache_features', array('get', 'post', 'put', 'delete')), 'is_string'), '#options' => array('get' => t('GET/HEAD'), 'post' => t('POST'), 'put' => t('PUT'), 'delete' => t('DELETE')), '#description' => t('Determines which HTTP methods will be supported in the REST API. For finer-grained access controls based on user credentials, refer to the Drupal <a href="@manage-perms">permissions</a> for Bitcache.', array('@manage-perms' => url('admin/user/permissions', array('fragment' => 'module-bitcache')))));
  $form['server']['bitcache_transfer_method'] = array('#type' => 'select', '#title' => t('Download method'), '#default_value' => BITCACHE_TRANSFER_METHOD, '#options' => array('redirect' => t('Redirect (external)'), 'xsendfile' => t('Redirect (internal X-Sendfile)'), 'readfile' => t('Passthrough (unbuffered)'), 'read4k' => t('Passthrough (4K chunks)')), '#description' => t('Determines how the bitstream contents will be transferred to the client when a download is requested. Note that this is a default setting only, and the actual transfer method may in practice be determined by the repository storage location.'));

  // Notification settings
  //$form['events'] = array('#type' => 'fieldset', '#title' => t('Notification settings'), '#collapsible' => TRUE, '#collapsed' => TRUE); // TODO

  // Synchronization settings
  $form['syncing'] = array('#type' => 'fieldset', '#title' => t('Synchronization settings'), '#collapsible' => TRUE, '#collapsed' => TRUE);
  $form['syncing']['bitcache_cron'] = array('#type' => 'textfield', '#title' => t('Shell command to execute on cron runs'), '#default_value' => BITCACHE_CRON, '#maxlength' => 255, '#description' => t('To synchronize the local Bitcache repository with a remote location, enter an <em>rsync</em> or <em>bitsync</em> command that should be triggered during cron maintenance runs. Leave empty to disable this functionality.'), '#required' => FALSE);
  // TODO: execute only if creates/deletes vs. execute regardless

  // Stream wrappers
  $wrappers = implode(', ', stream_get_wrappers()); // requires PHP 5
  $form['wrappers'] = array('#type' => 'fieldset', '#title' => t('Stream wrappers'), '#collapsible' => TRUE, '#collapsed' => TRUE);
  $form['wrappers']['list'] = array('#value' => '<div class="form-item"><label>'. t('Available stream wrappers') .':</label>'. $wrappers .'<div class="description">'. t('This is the list of currently registered <a href="http://php.net/stream">PHP stream wrappers</a> that can be used when specifying storage locations for <a href="@manage-repos">repositories</a>.', array('@manage-repos' => url('admin/settings/bitcache/repositories'))) .'</div></div>');

  return system_settings_form($form);
}

//////////////////////////////////////////////////////////////////////////////
// Bitcache bitstream management

function bitcache_admin_bitstreams() {
  drupal_add_css(drupal_get_path('module', 'bitcache') .'/bitcache.css');

  $head = array(t('ID'), t('Kind'), array('data' => t('Size'), 'class' => 'size'), array('data' => t('Operations'), 'colspan' => 2));
  $rows = array();

  $repo = bitcache_get_repository_proxy();
  foreach ($repo as $id => $stream) {
    $rows[$id] = array(
      l($id, 'admin/content/bitcache/view/'. $id),
      $stream->type,
      array('data' => format_size($stream->size), 'class' => 'size'),
      l('download', 'bitcache/'. $id),
      l('delete', 'admin/content/bitcache/delete/'. $id),
    );
  }
  ksort($rows);
  $rows = array_values($rows);

  if (empty($rows)) {
    $rows[] = array(array('data' => t('No bitstreams available.'), 'colspan' => '5'));
  }

  return theme('table', $head, $rows, array('class' => 'bitcache'));
}

function bitcache_admin_bitstream_upload() {
  $form = array();

  $form['upload'] = array('#type' => 'fieldset', '#title' => t('Upload file'));
  $form['upload']['attach'] = array(
    '#type' => 'file', 
    '#title' => '',
    '#description' => t(''), // TODO
  );

  $form['submit'] = array('#type' => 'submit', '#value' => t('Upload bitstream'));
  $form['#attributes']['enctype'] = 'multipart/form-data';
  return $form;
}

function bitcache_admin_bitstream_upload_validate($form, &$form_state) {
  extract($form_state['values'], EXTR_SKIP | EXTR_REFS);
  $source = 'attach';

  if (!isset($_FILES['files']) || !$_FILES['files']['name'][$source] || !is_uploaded_file($_FILES['files']['tmp_name'][$source])) {
    form_set_error($source, t('File upload required.'));
  }
}

function bitcache_admin_bitstream_upload_submit($form, &$form_state) {
  extract($form_state['values'], EXTR_SKIP | EXTR_REFS);
  $source = 'attach';

  if (isset($_FILES['files']) && $_FILES['files']['name'][$source] && is_uploaded_file($_FILES['files']['tmp_name'][$source])) {
    if (!($id = bitcache_put_file(NULL, $_FILES['files']['tmp_name'][$source], TRUE))) {
      watchdog('bitcache', 'Upload error. Could not move uploaded file %file to repository.', array('%file' => $_FILES['files']['tmp_name'][$source]), WATCHDOG_ERROR);
      return;
    }
    watchdog('bitcache', 'Bitstream uploaded: %id.', array('%id' => $id), WATCHDOG_NOTICE, l(t('view'), 'admin/content/bitcache'));
    drupal_set_message(t('The bitstream %id was successfully uploaded.', array('%id' => $id)));
  }

  $form_state['redirect'] = 'admin/content/bitcache';
}

function bitcache_admin_bitstream_fetch() {
  $form = array();

  $form['fetch'] = array('#type' => 'fieldset', '#title' => t('Fetch URL'));
  $form['fetch']['url'] = array('#type' => 'textfield', '#title' => '', '#default_value' => 'http://', '#maxlength' => 255, '#description' => t(''), '#required' => FALSE);

  $form['submit'] = array('#type' => 'submit', '#value' => t('Fetch bitstream'));
  return $form;
}

function bitcache_admin_bitstream_fetch_validate($form, &$form_state) {
  extract($form_state['values'], EXTR_SKIP | EXTR_REFS);

  $url = trim($url);
  if (!valid_url($url) || !@parse_url($url)) {
    form_set_error('url', t('Malformed or invalid URL.'));
  }
}

function bitcache_admin_bitstream_fetch_submit($form, &$form_state) {
  extract($form_state['values'], EXTR_SKIP | EXTR_REFS);

  if ($response = @drupal_http_request($url)) {
    if (isset($response->error)) {
      form_set_error('url', t('Error fetching %url: @error', array('%url' => $url, '@error' => $response->error)));
      return;
    }

    if (!($id = bitcache_put(NULL, $response->data))) {
      watchdog('bitcache', 'Upload error. Could not store contents of %url to repository.', array('%url' => $url), WATCHDOG_ERROR);
      return;
    }
    watchdog('bitcache', 'Bitstream uploaded: %id.', array('%id' => $id), WATCHDOG_NOTICE, l(t('view'), 'admin/content/bitcache'));
    drupal_set_message(t('The bitstream %id was successfully fetched.', array('%id' => $id)));
  }

  $form_state['redirect'] = 'admin/content/bitcache';
}

function bitcache_admin_bitstream_view($id) {
  if (!bitcache_exists($id)) {
    return drupal_not_found();
  }

  $blob = bitcache_get($id);

  $rows = array(
    array(
      array('data' => t('Repository'), 'header' => TRUE),
      t('Default'), // FIXME
    ),
    array(
      array('data' => t('ID'), 'header' => TRUE),
      t('@id (%algo)', array('@id' => $id, '%algo' => 'SHA-1')), // FIXME
    ),
    array(
      array('data' => t('Size'), 'header' => TRUE),
      format_size($blob->size),
    ),
    array(
      array('data' => t('MIME type'), 'header' => TRUE),
      check_plain($blob->type),
    ),
    array(
      array('data' => t('Created'), 'header' => TRUE),
      format_date(filectime($blob->path), 'large'),
    ),
    array(
      array('data' => t('Compressed'), 'header' => TRUE),
      !empty($blob->compressed) ? t('Yes') : t('No'),
    ),
    array(
      array('data' => t('Encrypted'), 'header' => TRUE),
      !empty($blob->encrypted) ? t('Yes') : t('No'),
    ),
  );

  return theme('table', array(), $rows, array('class' => 'rdfbus-message'));
}

function bitcache_admin_bitstream_delete($form_state, $id) {
  $form['id'] = array('#type' => 'value', '#value' => $id);

  return confirm_form($form,
    t('Are you sure you want to delete the bitstream %id?', array('%id' => $id)),
    isset($_GET['destination']) ? $_GET['destination'] : 'admin/content/bitcache',
    t('This action cannot be undone.'));
}

function bitcache_admin_bitstream_delete_submit($form, &$form_state) {
  extract($form_state['values'], EXTR_SKIP | EXTR_REFS);

  if ($form_state['values']['confirm']) {
    bitcache_delete($id);

    watchdog('bitcache', 'Bitstream deleted: %id.', array('%id' => $id));
    drupal_set_message(t('The bitstream %id has been deleted.', array('%id' => $id)));
  }

  $form_state['redirect'] = 'admin/content/bitcache';
}

//////////////////////////////////////////////////////////////////////////////
// Bitcache repository management

/**
 * Menu callback: displays the Bitcache repository management screen.
 */
function bitcache_admin_repos() {
  static $repo_types = array('file' => 'File system');

  $head = array(t('Name'), t('Kind'), t('Total Items'), t('Total Size'), array('data' => t('Operations'), 'colspan' => '2'));
  $rows = array();

  foreach (bitcache_get_repositories('titles') as $name => $title) {
    $mutable = ($name != 'default');
    $rows[] = array(
      $title,
      t($repo_types['file']),
      number_format(bitcache_get_total_count($name)),
      format_size(bitcache_get_total_size($name)),
      !$mutable ? '' : l(t('configure'), 'admin/settings/bitcache/repositories/edit/'. $name),
      !$mutable ? '' : l(t('delete'), 'admin/settings/bitcache/repositories/delete/'. $name),
    );
  }

  return theme('table', $head, $rows);
}

function bitcache_admin_repo_edit($name = '') {
  if (empty($name)) {
    return drupal_get_form('bitcache_admin_repo_form');
  }
  else {
    return drupal_get_form('bitcache_admin_repo_form', bitcache_get_repository_settings($name));
  }
}

function bitcache_admin_repo_form(&$form_state, $edit = array('name' => '', 'title' => '', 'description' => '', 'location' => '')) {
  $edit = (object)$edit;
  $form = array();

  $form['identity'] = array('#type' => 'fieldset', '#title' => t('Identification'));
  $form['identity']['title'] = array('#type' => 'textfield', '#title' => t('Title'), '#default_value' => $edit->title, '#maxlength' => 64, '#description' => t('The human-readable name of this repository. This text will be displayed as part of the list on the repository management page. This name must begin with a capital letter and contain only letters, numbers, and spaces. This name must be unique.'), '#required' => TRUE, '#disabled' => ($edit->name == 'default'));
  $form['identity']['name'] = array('#type' => 'textfield', '#title' => t('Name'), '#default_value' => $edit->name, '#maxlength' => 64, '#description' => t('The machine-readable name of this repository. This name must contain only lowercase letters, numbers, and underscores. This name must be unique.'), '#required' => TRUE, '#disabled' => ($edit->name == 'default'));
  $form['identity']['description'] = array('#title' => t('Description'), '#type' => 'textarea', '#default_value' => $edit->description, '#rows' => 2, '#description' => t('A brief description of this repository.'), '#disabled' => ($edit->name == 'default'));

  $form['storage'] = array('#type' => 'fieldset', '#title' => t('Storage'));
  $form['storage']['location'] = array('#type' => 'textfield', '#title' => t('Location'), '#default_value' => $edit->location, '#maxlength' => 255, '#description' => t('The storage location for this repository, given either as a local file system path or a full URL making use of any of the available PHP stream wrappers.'), '#required' => TRUE);

  // TODO: Policy settings
    // Minimum bitstream size
    // Maximum bitstream size
  // TODO: Compression settings
  // TODO: Encryption settings

  $form['key'] = array('#type' => 'hidden', '#value' => $edit->name);
  $form['submit'] = array('#type' => 'submit', '#value' => empty($edit->name) ? t('Create new repository') : t('Update repository settings'));
  return $form;
}

function bitcache_admin_repo_form_validate($form, &$form_state) {
  extract($form_state['values'], EXTR_SKIP | EXTR_REFS);

  if (!preg_match('/^[a-z]+[a-z\d_]*$/', $name)) {
    form_set_error('name', t('The machine-readable name can only consist of lowercase letters, underscores, and numbers.', array('%name' => $name)));
  }

  if (empty($key) && array_search($name, bitcache_get_repositories('names')) !== FALSE) {
    form_set_error('name', t('The machine-readable name %name is already used by another repository.', array('%name' => $name)));
  }
}

function bitcache_admin_repo_form_submit($form, &$form_state) {
  extract($form_state['values'], EXTR_SKIP | EXTR_REFS);

  if (empty($key)) {
    bitcache_create_repository($name, compact('title', 'description', 'location'));
    drupal_set_message(t('The repository has been created.'));
  }
  else {
    bitcache_update_repository($name, compact('title', 'description', 'location'));
    drupal_set_message(t('The repository settings have been updated.'));
  }

  $form_state['redirect'] = 'admin/settings/bitcache/repositories';
}

function bitcache_admin_repo_delete($form_state, $name) {
  $form['name'] = array('#type' => 'value', '#value' => $name);
  $output = confirm_form($form,
    t('Are you sure you want to delete the repository %title?', array('%title' => $name)),
    isset($_GET['destination']) ? $_GET['destination'] : 'admin/settings/bitcache/repositories',
    t('This action will destroy all data contained in the repository and cannot be undone.'));
  return $output;
}

function bitcache_admin_repo_delete_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    bitcache_delete_repository($form_state['values']['name']);
    drupal_set_message(t('The repository has been deleted.'));

    $form_state['redirect'] = 'admin/settings/bitcache/repositories';
  }
}
