<?php
// $Id: bitcache.server.inc,v 1.8 2008/10/01 08:32:52 arto Exp $

//////////////////////////////////////////////////////////////////////////////

module_load_include('inc', 'bitcache');

//////////////////////////////////////////////////////////////////////////////
// Menu callbacks

function bitcache_server() {
  // The REST API must be explicitly enabled by the administrator:
  if (!BITCACHE_SERVER) {
    die(drupal_access_denied());
  }

  $method  = $_SERVER['REQUEST_METHOD'];
  $request = $_SERVER['REDIRECT_URL']; // FIXME: handle lack of mod_rewrite?

  // Create the repository directory if it doesn't exist yet.
  file_check_directory($path = BITCACHE_ROOT, TRUE);

  // Attempt to dispatch the request to the Bitcache server implementation.
  $base = url(BITCACHE_ALIAS, array('alias' => TRUE));
  if (strpos($request, $base) === 0) {
    $request = substr($request, strlen($base));

    // Ensure that the bitstream index is accessed with a trailing slash.
    if (!$request) {
      die(drupal_goto(BITCACHE_ALIAS .'/', NULL, NULL, 301));
    }

    $server = new Bitcache_DrupalServer(
      bitcache_get_repository_proxy(),
      array('tmpdir' => file_directory_temp())
    );
    $server->$method($request);
    die();
  }

  // A path alias has been defined for the Bitcache server root URL, but the
  // incoming request was still using an internal path; redirect the client
  // to the correct address.
  $base = url(BITCACHE_BASE, array('alias' => TRUE));
  if (strpos($request, $base) === 0) {
    $request = substr($request, strlen($base));
    die(drupal_goto(BITCACHE_ALIAS . $request ? $request : '/', NULL, NULL, 301));
  }

  die(drupal_not_found()); // should never get here
}

//////////////////////////////////////////////////////////////////////////////
// Drupal server implementation

class Bitcache_DrupalServer extends Bitcache_Server {

  function index($request, array $query = array(), $body = TRUE) {
    static $formats = array(
      'text/plain'       => 'text',
      //'text/html'        => 'html', // TODO
      'application/json' => 'json',
      'text/x-ntriples'  => 'ntriples',
    );

    // Content negotiation based on the HTTP Accept header and, secondarily,
    // on the ?format= URL query parameter.
    $format = isset($_GET['format']) ? $_GET['format'] : NULL;
    foreach (explode(',', $_SERVER['HTTP_ACCEPT']) as $accept) {
      list($mime, ) = explode(';', $accept);
      if (isset($formats[$mime])) {
        $format = $mime;
        break;
      }
    }
    $format = isset($_GET['format']) ? $_GET['format'] : $format;
    $format = isset($formats[$format]) ? $format : 'text/plain';

    $this->header('Content-Type: '. $format .'; charset=ascii');

    if ($body) {
      $handler = 'index_'. $formats[$format];
      $this->$handler();
    }

    return TRUE;
  }

  protected function index_json() {
    $index = array();
    foreach ($this->repo as $id => $stream) {
      $index[$id] = $stream->size;
    }
    print drupal_to_js($index);
  }

  protected function index_ntriples() {
    foreach ($this->repo as $id => $stream) {
      $uri = url('bitcache/'. $id, array('absolute' => TRUE));
      printf("<%s> <%s> \"%d\"^^<http://www.w3.org/2001/XMLSchema#integer> .\n",
        $uri, 'http://purl.org/dc/terms/extent', $stream->size);
    }
  }

  protected function allow($method, $id = NULL) {
    switch ($method) {
      case 'GET':
        foreach (module_implements('bitcache') as $module) {
          if (($result = module_invoke($module, 'bitcache', 'access', $id)) !== FALSE) {
            if (!is_null($result)) {
              return $result;
            }
          }
        }
        break;
    }
    return parent::allow($method, $id);
  }

  protected function headers($id, $stream) {
    parent::headers($id, $stream);
    foreach (module_invoke_all('bitcache', 'download', $id, $stream) as $key => $value) {
      $this->header($key .': '. $value);
    }
  }

  protected function created($id, $stream) {
    watchdog('bitcache', 'Bitstream created: %id.', array('%id' => $id), WATCHDOG_NOTICE, l(t('view'), 'admin/content/bitcache/view/'. $id));
    parent::created($id);
    module_invoke_all('bitcache', 'insert', $id, $stream);
  }

  protected function deleted($id) {
    watchdog('bitcache', 'Bitstream deleted: %id.', array('%id' => $id), WATCHDOG_NOTICE, l(t('view'), 'admin/content/bitcache'));
    parent::deleted($id);
    module_invoke_all('bitcache', 'delete', $id);
  }

}
